==> lein-template-0.6.3.jar: leiningen/new/jurassic_web/src/utilities/testing.clj <==
(ns utilities.testing
  (:require
    [clojure.tools.namespace.find :as find]
    [clojure.test :refer [run-tests]]
    [utilities.db :as db]
    [utilities.settings :as settings]
    [utilities.migrations :as m]
    [clojure.java.jdbc :as jdbc]
    [clojure.java.io :as io]
    [clojure.string :as s]
    [peridot.core :as peridot]
    [utilities.exceptions :as ex]))


(def ^:dynamic *session*  nil)


(defn request
  "Mock request in a session"
  ([method uri] (request method uri nil))

  ([method uri params]
   (when-not *session*
     (ex/raise :no-session "Please create a session"))

   (peridot/request *session*
                    uri
                    :request-method method
                    :params params)))

(defn new-session
  "Creates a new peridot session"
  [app user]
  (let [session   (peridot/session app)]
    (if user
      (binding [*session* session]
        (request :post "/login/" user))
      session)))


(defn- run-on-all-tables
  "Runs a command on all tables in database"
  [command]
  (let [tables (jdbc/query (settings/db-spec)
                           ["SELECT table_name
                             FROM information_schema.tables
                             WHERE table_schema = ?" settings/*mysql-db*])
        user-commands (map (fn [{:keys [table_name]}]
                             (str command " " table_name))
                           tables)
        all-commands (concat ["SET FOREIGN_KEY_CHECKS = 0"]
                             user-commands
                             ["SET FOREIGN_KEY_CHECKS=1"])]
    (println "ALL COMMANDS" all-commands)
    (jdbc/db-do-commands (settings/db-spec) all-commands)))


(defn rollback
  "Rolls-back everything by running tests inside a transaction"
  [test]
  (jdbc/with-db-transaction [t-conn (settings/db-spec)]
                            (jdbc/db-set-rollback-only! t-conn)
                            (binding [db/*connection* (delay t-conn)
                                      db/*connection* (delay t-conn)]
                              (test))))


(defn reset-db
  "Drops all tables in database"
  []
  (run-on-all-tables "DROP TABLE IF EXISTS"))


(defn truncate
  "Truncate all tables"
  ([]
   (truncate (fn [] (println "Truncated all data"))))
  ([test]
   (run-on-all-tables "TRUNCATE TABLE")
   (test)))


(defn redirects?
  "Returns true if the response redirects to given location"
  [res loc]
  (let [uri   (get (:headers res) "Location")
        r-loc (str "http://localhost" loc)]
    (or (= loc uri) (= r-loc uri))))


(defn run
  "Parse argument for ns name
  if :reset-db in arguments, then reset-db before test
  Wont check migrations if :re-test"
  ([] (run ""))
  ([pattern]
   (binding [settings/*in-testing?*     true
             settings/*in-production?*  true
             settings/*mysql-db*        (str "test_" settings/*mysql-db*)]
     ; run migrations
     (let [pending-m  (m/pending-count)]
       (when (> pending-m 0)
         (println "RUNNING " pending-m " pending migrations")
         (m/migrate)))
     ; load urls
     (require '[paper-books.urls])
     ; load tests in given pattern
     (let [ns (find/find-namespaces-in-dir (io/file "test"))
           ns (filter #(s/includes? (str %) pattern) ns)
           ns (map (fn [ns] (require ns) (find-ns ns)) ns)]
       (apply run-tests ns)))))

==> lein-template-0.7.4.jar: leiningen/new/jurassic_web/src/utilities/testing.clj <==
(ns utilities.testing
  (:require
    [clojure.tools.namespace.find :as find]
    [clojure.test :refer [run-tests]]
    [utilities.db :as db]
    [utilities.settings :as settings]
    [utilities.migrations :as m]
    [clojure.java.jdbc :as jdbc]
    [clojure.java.io :as io]
    [clojure.string :as s]))


(defn- run-on-all-tables
  "Runs a command on all tables in database"
  [command]
  (let [tables (jdbc/query (settings/db-spec)
                           ["SELECT table_name
                             FROM information_schema.tables
                             WHERE table_schema = ?" settings/*mysql-db*])
        user-commands (map (fn [{:keys [table_name]}]
                             (str command " " table_name))
                           tables)
        all-commands (concat ["SET FOREIGN_KEY_CHECKS = 0"]
                             user-commands
                             ["SET FOREIGN_KEY_CHECKS=1"])]
    (println "ALL COMMANDS" all-commands)
    (jdbc/db-do-commands (settings/db-spec) all-commands)))


(defn rollback
  "Rolls-back everything by running tests inside a transaction"
  [test]
  (jdbc/with-db-transaction [t-conn (settings/db-spec)]
                            (jdbc/db-set-rollback-only! t-conn)
                            (binding [db/*connection* (delay t-conn)
                                      db/*connection* (delay t-conn)]
                              (test))))


(defn reset-db
  "Drops all tables in database"
  []
  (run-on-all-tables "DROP TABLE IF EXISTS"))


(defn truncate
  "Truncate all tables"
  ([]
   (truncate (fn [] (println "Truncated all data"))))
  ([test]
   (run-on-all-tables "TRUNCATE TABLE")
   (test)))


(defn redirects?
  "Returns true if the response redirects to given location"
  [res loc]
  (let [uri   (get (:headers res) "Location")
        r-loc (str "http://localhost" loc)]
    (or (= loc uri) (= r-loc uri))))


(defn run
  "Parse argument for ns name
  if :reset-db in arguments, then reset-db before test
  Wont check migrations if :re-test"
  ([] (run ""))
  ([pattern & options]
   (let [options  (set options)
         reset?   (contains? options ":reset")]
     (binding [settings/*in-testing?*     true
               settings/*in-production?*  true
               settings/*mysql-db*        (str "test_" settings/*mysql-db*)]

       ; run migrations
       (when reset?
         (let [pending-m  (m/pending-count)]
           (when (> pending-m 0)
             (println "RUNNING " pending-m " pending migrations")
             (m/migrate))))

       ; load urls and app settings
       (require '[screener.handler])

       ; load tests in given pattern
       (let [ns (find/find-namespaces-in-dir (io/file "test"))
             ns (filter #(s/includes? (str %) pattern) ns)
             ns (map (fn [ns] (require ns) (find-ns ns)) ns)]
         (apply run-tests ns))))))
